module hecuba_exporter_lib
    use iso_c_binding

    private
    public :: hecuba_exporter

    ! Yes, include is a keyword in Fortran !
    include 'hecuba_exporter_cdef.f90'

    ! We'll use a Fortan type to represent a C++ class here, in an opaque maner
    type hecuba_exporter
        private
        type(c_ptr) :: ptr ! pointer to the Hecuba_exporter class
    contains
        ! We can bind some functions to this type, allowing for a cleaner syntax.
#ifdef __GNUC__
        procedure :: delete => delete_hecuba_exporter_polymorph ! Destructor for gfortran
#else
        final :: delete_hecuba_exporter ! Destructor
#endif
        ! Function member
        procedure :: load_datamodel => load_datamodel_wrapper
    end type

    ! This function will act as the constructor for foo type
    interface hecuba_exporter
        procedure create_hecuba_exporter
    end interface

contains ! Implementation of the functions. We just wrap the C function here.
    function create_hecuba_exporter(a, b)
        implicit none
        type(hecuba_exporter) :: create_hecuba_exporter
        integer, intent(in) :: a, b
        create_hecuba_exporter%ptr = create_hecuba_exporter_c(a, b)
    end function

    subroutine delete_hecuba_exporter(this)
        implicit none
        type(hecuba_exporter) :: this
        call delete_hecuba_exporter_c(this%ptr)
    end subroutine

    ! Bounds procedure needs to take a polymorphic (class) argument
    subroutine delete_hecuba_exporter_polymorph(this)
        implicit none
        class(hecuba_exporter) :: this
        call delete_hecuba_exporter_c(this%ptr)
    end subroutine

    double precision function load_datamodel_wrapper(this, c)
        implicit none
        class(hecuba_exporter), intent(in) :: this
        double precision, intent(in) :: c
        load_datamodel_wrapper = hecuba_exporter_load_datamodel_c(this%ptr, c)
    end function
end module
